# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Decoder error
"""

NONE: int = 0
"""
No error. The last decoded instruction is a valid instruction
"""
INVALID_INSTRUCTION: int = 1
"""
It's an invalid instruction or an invalid encoding of an existing instruction (eg. some reserved bit is set/cleared)
"""
NO_MORE_BYTES: int = 2
"""
There's not enough bytes left to decode the instruction
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
The code size (16/32/64) that was used when an instruction was decoded
"""

UNKNOWN: int = 0
"""
Unknown size
"""
CODE16: int = 1
"""
16-bit code
"""
CODE32: int = 2
"""
32-bit code
"""
CODE64: int = 3
"""
64-bit code
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JNP`` / ``JPO``)
"""

NP: int = 0
"""
``JNP``, ``CMOVNP``, ``SETNP``
"""
PO: int = 1
"""
``JPO``, ``CMOVPO``, ``SETPO``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

from iced_x86 import *

def test_default_options():
	formatter = FastFormatter()
	assert not formatter.space_after_operand_separator
	assert not formatter.rip_relative_addresses
	assert formatter.use_pseudo_ops
	assert not formatter.show_symbol_address
	assert not formatter.always_show_segment_register
	assert not formatter.always_show_memory_size
	assert formatter.uppercase_hex
	assert not formatter.use_hex_prefix

	formatter.space_after_operand_separator = not formatter.space_after_operand_separator
	formatter.rip_relative_addresses = not formatter.rip_relative_addresses
	formatter.use_pseudo_ops = not formatter.use_pseudo_ops
	formatter.show_symbol_address = not formatter.show_symbol_address
	formatter.always_show_segment_register = not formatter.always_show_segment_register
	formatter.always_show_memory_size = not formatter.always_show_memory_size
	formatter.uppercase_hex = not formatter.uppercase_hex
	formatter.use_hex_prefix = not formatter.use_hex_prefix

	assert formatter.space_after_operand_separator
	assert formatter.rip_relative_addresses
	assert not formatter.use_pseudo_ops
	assert formatter.show_symbol_address
	assert formatter.always_show_segment_register
	assert formatter.always_show_memory_size
	assert not formatter.uppercase_hex
	assert formatter.use_hex_prefix

def test_format():
	decoder = Decoder(64, b"\x62\xF2\x4F\xDD\x72\x50\x03")
	instr = decoder.decode()
	formatter = FastFormatter()

	assert type(formatter.format(instr)) == str

	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z},zmm6,dword bcst [rax+0Ch]"
	formatter.space_after_operand_separator = True
	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z}, zmm6, dword bcst [rax+0Ch]"
	formatter.use_hex_prefix = True
	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z}, zmm6, dword bcst [rax+0xC]"
	formatter.always_show_segment_register = True
	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z}, zmm6, dword bcst ds:[rax+0xC]"
	formatter.uppercase_hex = False
	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z}, zmm6, dword bcst ds:[rax+0xc]"



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JBE`` / ``JNA``)
"""

BE: int = 0
"""
``JBE``, ``CMOVBE``, ``SETBE``
"""
NA: int = 1
"""
``JNA``, ``CMOVNA``, ``SETNA``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import pytest
from iced_x86 import *

def test_memory_size_ext():
	assert MemorySizeExt.size(MemorySize.UINT128) == 16
	assert MemorySizeExt.element_size(MemorySize.UINT128) == 16
	assert MemorySizeExt.element_type(MemorySize.UINT128) == MemorySize.UINT128
	assert not MemorySizeExt.is_signed(MemorySize.UINT128)
	assert MemorySizeExt.is_signed(MemorySize.PACKED256_INT16)
	assert not MemorySizeExt.is_packed(MemorySize.UINT128)
	assert MemorySizeExt.element_count(MemorySize.UINT128) == 1
	assert not MemorySizeExt.is_broadcast(MemorySize.UINT128)

	assert MemorySizeExt.size(MemorySize.PACKED256_INT16) == 32
	assert MemorySizeExt.element_size(MemorySize.PACKED256_INT16) == 2
	assert MemorySizeExt.element_type(MemorySize.PACKED256_INT16) == MemorySize.INT16
	assert MemorySizeExt.is_signed(MemorySize.PACKED256_INT16)
	assert MemorySizeExt.is_packed(MemorySize.PACKED256_INT16)
	assert MemorySizeExt.element_count(MemorySize.PACKED256_INT16) == 16
	assert not MemorySizeExt.is_broadcast(MemorySize.PACKED256_INT16)

	assert MemorySizeExt.is_broadcast(MemorySize.BROADCAST128_2X_INT32)

	assert MemorySizeExt.info(MemorySize.PACKED256_INT16).memory_size == MemorySize.PACKED256_INT16
	assert MemorySizeExt.info(MemorySize.PACKED256_INT16).size == 32
	assert MemorySizeExt.element_type_info(MemorySize.PACKED256_INT16).element_type == MemorySize.INT16

@pytest.mark.parametrize("create", [
	lambda memory_size: MemorySizeExt.info(memory_size),
	lambda memory_size: MemorySizeInfo(memory_size),
])
def test_memory_size_info(create):
	info = create(MemorySize.UINT128)
	assert info.memory_size == MemorySize.UINT128
	assert info.size == 16
	assert info.element_size == 16
	assert info.element_type == MemorySize.UINT128
	assert not info.is_signed
	assert create(MemorySize.PACKED256_INT16).is_signed
	assert not info.is_packed
	assert info.element_count == 1
	assert not info.is_broadcast

	info = create(MemorySize.PACKED256_INT16)
	assert info.memory_size == MemorySize.PACKED256_INT16
	assert info.size == 32
	assert info.element_size == 2
	assert info.element_type == MemorySize.INT16
	assert info.is_signed
	assert info.is_packed
	assert info.element_count == 16
	assert not info.is_broadcast

	assert create(MemorySize.BROADCAST128_2X_INT32).is_broadcast

@pytest.mark.parametrize("create", [
	lambda memory_size: MemorySizeExt.info(memory_size),
	lambda memory_size: MemorySizeInfo(memory_size),
])
def test_memory_size_info_invalid_arg(create):
	with pytest.raises(ValueError):
		create(1234)

def test_ext_size_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.size(1234)

def test_ext_element_size_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.element_size(1234)

def test_ext_element_type_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.element_type(1234)

def test_ext_element_type_info_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.element_type_info(1234)

def test_ext_is_signed_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.is_signed(1234)

def test_ext_is_packed_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.is_packed(1234)

def test_ext_element_count_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.element_count(1234)

def test_ext_is_broadcast_invalid_arg():
	with pytest.raises(ValueError):
		MemorySizeExt.is_broadcast(1234)



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# pylint: disable=line-too-long
# pylint: disable=no-name-in-module

"""
iced-x86 is a blazing fast and correct x86/x64 disassembler, assembler and instruction decoder written in Rust with Python bindings
"""

from ._iced_x86_py import BlockEncoder, ConstantOffsets, Decoder, Encoder, FastFormatter, \
	Formatter, FpuStackIncrementInfo, Instruction, InstructionInfo, \
	InstructionInfoFactory, MemoryOperand, MemorySizeExt, MemorySizeInfo, OpCodeInfo, \
	RegisterExt, RegisterInfo, UsedMemory, UsedRegister

from . import CC_a
from . import CC_ae
from . import CC_b
from . import CC_be
from . import CC_e
from . import CC_g
from . import CC_ge
from . import CC_l
from . import CC_le
from . import CC_ne
from . import CC_np
from . import CC_p
from . import Code
from . import CodeSize
from . import ConditionCode
from . import CpuidFeature
from . import DecoderError
from . import DecoderOptions
from . import EncodingKind
from . import FlowControl
from . import FormatMnemonicOptions
from . import FormatterSyntax
from . import MandatoryPrefix
from . import MemorySize
from . import MemorySizeOptions
from . import Mnemonic
from . import OpAccess
from . import OpCodeOperandKind
from . import OpCodeTableKind
from . import OpKind
from . import Register
from . import RepPrefixKind
from . import RflagsBits
from . import RoundingControl
from . import TupleType

__all__ = [
	"BlockEncoder",
	"ConstantOffsets",
	"Decoder",
	"Encoder",
	"FastFormatter",
	"Formatter",
	"FpuStackIncrementInfo",
	"Instruction",
	"InstructionInfo",
	"InstructionInfoFactory",
	"MemoryOperand",
	"MemorySizeExt",
	"MemorySizeInfo",
	"OpCodeInfo",
	"RegisterExt",
	"RegisterInfo",
	"UsedMemory",
	"UsedRegister",

	"CC_a",
	"CC_ae",
	"CC_b",
	"CC_be",
	"CC_e",
	"CC_g",
	"CC_ge",
	"CC_l",
	"CC_le",
	"CC_ne",
	"CC_np",
	"CC_p",
	"Code",
	"CodeSize",
	"ConditionCode",
	"CpuidFeature",
	"DecoderError",
	"DecoderOptions",
	"EncodingKind",
	"FlowControl",
	"FormatMnemonicOptions",
	"FormatterSyntax",
	"MandatoryPrefix",
	"MemorySize",
	"MemorySizeOptions",
	"Mnemonic",
	"OpAccess",
	"OpCodeOperandKind",
	"OpCodeTableKind",
	"OpKind",
	"Register",
	"RepPrefixKind",
	"RflagsBits",
	"RoundingControl",
	"TupleType",
]



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JAE`` / ``JNB`` / ``JNC``)
"""

AE: int = 0
"""
``JAE``, ``CMOVAE``, ``SETAE``
"""
NB: int = 1
"""
``JNB``, ``CMOVNB``, ``SETNB``
"""
NC: int = 2
"""
``JNC``, ``CMOVNC``, ``SETNC``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JA`` / ``JNBE``)
"""

A: int = 0
"""
``JA``, ``CMOVA``, ``SETA``
"""
NBE: int = 1
"""
``JNBE``, ``CMOVNBE``, ``SETNBE``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

from iced_x86 import *

FORMATTER_SYNTAX = FormatterSyntax.NASM

def test_default_options():
	formatter = Formatter(FORMATTER_SYNTAX)

	assert not formatter.uppercase_prefixes
	assert not formatter.uppercase_mnemonics
	assert not formatter.uppercase_registers
	assert not formatter.uppercase_keywords
	assert not formatter.uppercase_decorators
	assert not formatter.uppercase_all
	assert formatter.first_operand_char_index == 0
	assert formatter.tab_size == 0
	assert not formatter.space_after_operand_separator
	assert not formatter.space_after_memory_bracket
	assert not formatter.space_between_memory_add_operators
	assert not formatter.space_between_memory_mul_operators
	assert not formatter.scale_before_index
	assert not formatter.always_show_scale
	assert not formatter.always_show_segment_register
	assert not formatter.show_zero_displacements
	assert formatter.hex_prefix == ""
	assert formatter.hex_suffix == "h"
	assert formatter.hex_digit_group_size == 4
	assert formatter.decimal_prefix == ""
	assert formatter.decimal_suffix == ""
	assert formatter.decimal_digit_group_size == 3
	assert formatter.octal_prefix == ""
	assert formatter.octal_suffix == "o"
	assert formatter.octal_digit_group_size == 4
	assert formatter.binary_prefix == ""
	assert formatter.binary_suffix == "b"
	assert formatter.binary_digit_group_size == 4
	assert formatter.digit_separator == ""
	assert not formatter.leading_zeroes
	assert formatter.uppercase_hex
	assert formatter.small_hex_numbers_in_decimal
	assert formatter.add_leading_zero_to_hex_numbers
	assert formatter.number_base == 16
	assert formatter.branch_leading_zeroes
	assert not formatter.signed_immediate_operands
	assert formatter.signed_memory_displacements
	assert not formatter.displacement_leading_zeroes
	assert formatter.memory_size_options == MemorySizeOptions.DEFAULT
	assert not formatter.rip_relative_addresses
	assert formatter.show_branch_size
	assert formatter.use_pseudo_ops
	assert not formatter.show_symbol_address
	assert not formatter.gas_naked_registers
	assert not formatter.gas_show_mnemonic_size_suffix
	assert not formatter.gas_space_after_memory_operand_comma
	assert formatter.masm_add_ds_prefix32
	assert formatter.masm_symbol_displ_in_brackets
	assert formatter.masm_displ_in_brackets
	assert not formatter.nasm_show_sign_extended_immediate_size
	assert not formatter.prefer_st0
	assert not formatter.show_useless_prefixes
	assert formatter.cc_b == CC_b.B
	assert formatter.cc_ae == CC_ae.AE
	assert formatter.cc_e == CC_e.E
	assert formatter.cc_ne == CC_ne.NE
	assert formatter.cc_be == CC_be.BE
	assert formatter.cc_a == CC_a.A
	assert formatter.cc_p == CC_p.P
	assert formatter.cc_np == CC_np.NP
	assert formatter.cc_l == CC_l.L
	assert formatter.cc_ge == CC_ge.GE
	assert formatter.cc_le == CC_le.LE
	assert formatter.cc_g == CC_g.G

def test_format():
	instr, instr2 = [instr for instr in Decoder(64, b"\x62\xF2\x4F\xDD\x72\x50\x01\xF0\x00\x18")]
	formatter = Formatter(FORMATTER_SYNTAX)

	assert formatter.format(instr) == "vcvtne2ps2bf16 zmm2{k5}{z},zmm6,[rax+4]{1to16}"

	assert formatter.format_mnemonic(instr) == "vcvtne2ps2bf16"
	assert formatter.format_mnemonic(instr, FormatMnemonicOptions.NONE) == "vcvtne2ps2bf16"
	assert formatter.format_mnemonic(instr, FormatMnemonicOptions.NO_MNEMONIC) == ""
	assert formatter.format_mnemonic(instr, FormatMnemonicOptions.NO_PREFIXES) == "vcvtne2ps2bf16"
	assert formatter.format_mnemonic(instr, FormatMnemonicOptions.NO_MNEMONIC | FormatMnemonicOptions.NO_PREFIXES) == ""

	assert formatter.format_mnemonic(instr2) == "lock add"
	assert formatter.format_mnemonic(instr2, FormatMnemonicOptions.NONE) == "lock add"
	assert formatter.format_mnemonic(instr2, FormatMnemonicOptions.NO_MNEMONIC) == "lock"
	assert formatter.format_mnemonic(instr2, FormatMnemonicOptions.NO_PREFIXES) == "add"
	assert formatter.format_mnemonic(instr2, FormatMnemonicOptions.NO_MNEMONIC | FormatMnemonicOptions.NO_PREFIXES) == ""

	assert formatter.operand_count(instr) == 3
	assert formatter.op_access(instr, 0) is None
	assert formatter.get_instruction_operand(instr, 0) == 0
	assert formatter.get_instruction_operand(instr, 1) == 1
	assert formatter.get_instruction_operand(instr, 2) == 2
	assert formatter.get_formatter_operand(instr, 0) == 0
	assert formatter.get_formatter_operand(instr, 1) == 1
	assert formatter.get_formatter_operand(instr, 2) == 2
	assert formatter.format_operand(instr, 0) == "zmm2{k5}{z}"
	assert formatter.format_operand(instr, 1) == "zmm6"
	assert formatter.format_operand(instr, 2) == "[rax+4]{1to16}"
	assert formatter.format_operand_separator(instr) == ","
	assert formatter.format_all_operands(instr) == "zmm2{k5}{z},zmm6,[rax+4]{1to16}"

	assert formatter.format_register(Register.RCX) == "rcx"

	assert formatter.format_i8(-0x12) == "-12h"
	assert formatter.format_i16(-0x1234) == "-1234h"
	assert formatter.format_i32(-0x12345678) == "-12345678h"
	assert formatter.format_i64(-0x123456789ABCDEF0) == "-123456789ABCDEF0h"
	assert formatter.format_u8(0x89) == "89h"
	assert formatter.format_u16(0x89AB) == "89ABh"
	assert formatter.format_u32(0x89ABCDEF) == "89ABCDEFh"
	assert formatter.format_u64(0xFEDCBA9876543210) == "0FEDCBA9876543210h"



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
``CPUID`` feature flags
"""

INTEL8086: int = 0
"""
8086 or later
"""
INTEL8086_ONLY: int = 1
"""
8086 only
"""
INTEL186: int = 2
"""
80186 or later
"""
INTEL286: int = 3
"""
80286 or later
"""
INTEL286_ONLY: int = 4
"""
80286 only
"""
INTEL386: int = 5
"""
80386 or later
"""
INTEL386_ONLY: int = 6
"""
80386 only
"""
INTEL386_A0_ONLY: int = 7
"""
80386 A0-B0 stepping only (``XBTS``, ``IBTS`` instructions)
"""
INTEL486: int = 8
"""
Intel486 or later
"""
INTEL486_A_ONLY: int = 9
"""
Intel486 A stepping only (``CMPXCHG``)
"""
UMOV: int = 10
"""
UMOV (80386 and Intel486)
"""
IA64: int = 11
"""
IA-64
"""
X64: int = 12
"""
: CPUID.80000001H:EDX.LM[bit 29]
"""
ADX: int = 13
"""
: CPUID.(EAX=07H, ECX=0H):EBX.ADX[bit 19]
"""
AES: int = 14
"""
: CPUID.01H:ECX.AES[bit 25]
"""
AVX: int = 15
"""
: CPUID.01H:ECX.AVX[bit 28]
"""
AVX2: int = 16
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX2[bit 5]
"""
AVX512_4FMAPS: int = 17
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AVX512_4FMAPS[bit 3]
"""
AVX512_4VNNIW: int = 18
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AVX512_4VNNIW[bit 2]
"""
AVX512_BF16: int = 19
"""
: CPUID.(EAX=07H, ECX=1H):EAX.AVX512_BF16[bit 5]
"""
AVX512_BITALG: int = 20
"""
: CPUID.(EAX=07H, ECX=0H):ECX.AVX512_BITALG[bit 12]
"""
AVX512_IFMA: int = 21
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512_IFMA[bit 21]
"""
AVX512_VBMI: int = 22
"""
: CPUID.(EAX=07H, ECX=0H):ECX.AVX512_VBMI[bit 1]
"""
AVX512_VBMI2: int = 23
"""
: CPUID.(EAX=07H, ECX=0H):ECX.AVX512_VBMI2[bit 6]
"""
AVX512_VNNI: int = 24
"""
: CPUID.(EAX=07H, ECX=0H):ECX.AVX512_VNNI[bit 11]
"""
AVX512_VP2INTERSECT: int = 25
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AVX512_VP2INTERSECT[bit 08]
"""
AVX512_VPOPCNTDQ: int = 26
"""
: CPUID.(EAX=07H, ECX=0H):ECX.AVX512_VPOPCNTDQ[bit 14]
"""
AVX512BW: int = 27
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512BW[bit 30]
"""
AVX512CD: int = 28
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512CD[bit 28]
"""
AVX512DQ: int = 29
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512DQ[bit 17]
"""
AVX512ER: int = 30
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512ER[bit 27]
"""
AVX512F: int = 31
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512F[bit 16]
"""
AVX512PF: int = 32
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512PF[bit 26]
"""
AVX512VL: int = 33
"""
: CPUID.(EAX=07H, ECX=0H):EBX.AVX512VL[bit 31]
"""
BMI1: int = 34
"""
: CPUID.(EAX=07H, ECX=0H):EBX.BMI1[bit 3]
"""
BMI2: int = 35
"""
: CPUID.(EAX=07H, ECX=0H):EBX.BMI2[bit 8]
"""
CET_IBT: int = 36
"""
: CPUID.(EAX=07H, ECX=0H):EDX.CET_IBT[bit 20]
"""
CET_SS: int = 37
"""
: CPUID.(EAX=07H, ECX=0H):ECX.CET_SS[bit 7]
"""
CL1INVMB: int = 38
"""
``CL1INVMB`` instruction (Intel SCC = Single-Chip Computer)
"""
CLDEMOTE: int = 39
"""
: CPUID.(EAX=07H, ECX=0H):ECX.CLDEMOTE[bit 25]
"""
CLFLUSHOPT: int = 40
"""
: CPUID.(EAX=07H, ECX=0H):EBX.CLFLUSHOPT[bit 23]
"""
CLFSH: int = 41
"""
: CPUID.01H:EDX.CLFSH[bit 19]
"""
CLWB: int = 42
"""
: CPUID.(EAX=07H, ECX=0H):EBX.CLWB[bit 24]
"""
CLZERO: int = 43
"""
: CPUID.80000008H:EBX.CLZERO[bit 0]
"""
CMOV: int = 44
"""
: CPUID.01H:EDX.CMOV[bit 15]
"""
CMPXCHG16B: int = 45
"""
: CPUID.01H:ECX.CMPXCHG16B[bit 13]
"""
CPUID: int = 46
"""
``RFLAGS.ID`` can be toggled
"""
CX8: int = 47
"""
: CPUID.01H:EDX.CX8[bit 8]
"""
D3NOW: int = 48
"""
: CPUID.80000001H:EDX.3DNOW[bit 31]
"""
D3NOWEXT: int = 49
"""
: CPUID.80000001H:EDX.3DNOWEXT[bit 30]
"""
OSS: int = 50
"""
: CPUID.(EAX=12H, ECX=0H):EAX.OSS[bit 5]
"""
ENQCMD: int = 51
"""
: CPUID.(EAX=07H, ECX=0H):ECX.ENQCMD[bit 29]
"""
F16C: int = 52
"""
: CPUID.01H:ECX.F16C[bit 29]
"""
FMA: int = 53
"""
: CPUID.01H:ECX.FMA[bit 12]
"""
FMA4: int = 54
"""
: CPUID.80000001H:ECX.FMA4[bit 16]
"""
FPU: int = 55
"""
: 8087 or later (CPUID.01H:EDX.FPU[bit 0])
"""
FPU287: int = 56
"""
80287 or later
"""
FPU287XL_ONLY: int = 57
"""
80287XL only
"""
FPU387: int = 58
"""
80387 or later
"""
FPU387SL_ONLY: int = 59
"""
80387SL only
"""
FSGSBASE: int = 60
"""
: CPUID.(EAX=07H, ECX=0H):EBX.FSGSBASE[bit 0]
"""
FXSR: int = 61
"""
: CPUID.01H:EDX.FXSR[bit 24]
"""
CYRIX_D3NOW: int = 62
"""
Cyrix (AMD Geode GX/LX) 3DNow! instructions
"""
GFNI: int = 63
"""
: CPUID.(EAX=07H, ECX=0H):ECX.GFNI[bit 8]
"""
HLE: int = 64
"""
: CPUID.(EAX=07H, ECX=0H):EBX.HLE[bit 4]
"""
HLE_OR_RTM: int = 65
"""
:class:`HLE` or :class:`RTM`
"""
INVEPT: int = 66
"""
IA32_VMX_EPT_VPID_CAP[bit 20]
"""
INVPCID: int = 67
"""
: CPUID.(EAX=07H, ECX=0H):EBX.INVPCID[bit 10]
"""
INVVPID: int = 68
"""
IA32_VMX_EPT_VPID_CAP[bit 32]
"""
LWP: int = 69
"""
: CPUID.80000001H:ECX.LWP[bit 15]
"""
LZCNT: int = 70
"""
: CPUID.80000001H:ECX.LZCNT[bit 5]
"""
MCOMMIT: int = 71
"""
: CPUID.80000008H:EBX.MCOMMIT[bit 8]
"""
MMX: int = 72
"""
: CPUID.01H:EDX.MMX[bit 23]
"""
MONITOR: int = 73
"""
: CPUID.01H:ECX.MONITOR[bit 3]
"""
MONITORX: int = 74
"""
: CPUID.80000001H:ECX.MONITORX[bit 29]
"""
MOVBE: int = 75
"""
: CPUID.01H:ECX.MOVBE[bit 22]
"""
MOVDIR64B: int = 76
"""
: CPUID.(EAX=07H, ECX=0H):ECX.MOVDIR64B[bit 28]
"""
MOVDIRI: int = 77
"""
: CPUID.(EAX=07H, ECX=0H):ECX.MOVDIRI[bit 27]
"""
MPX: int = 78
"""
: CPUID.(EAX=07H, ECX=0H):EBX.MPX[bit 14]
"""
MSR: int = 79
"""
: CPUID.01H:EDX.MSR[bit 5]
"""
MULTIBYTENOP: int = 80
"""
: Multi-byte nops (``0F1F /0``): CPUID.01H.EAX[Bits 11:8] = 0110B or 1111B
"""
PADLOCK_ACE: int = 81
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.ACE[Bits 7:6] = 11B ([6] = exists, [7] = enabled)
"""
PADLOCK_PHE: int = 82
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.PHE[Bits 11:10] = 11B ([10] = exists, [11] = enabled)
"""
PADLOCK_PMM: int = 83
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.PMM[Bits 13:12] = 11B ([12] = exists, [13] = enabled)
"""
PADLOCK_RNG: int = 84
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.RNG[Bits 3:2] = 11B ([2] = exists, [3] = enabled)
"""
PAUSE: int = 85
"""
``PAUSE`` instruction (Pentium 4 or later)
"""
PCLMULQDQ: int = 86
"""
: CPUID.01H:ECX.PCLMULQDQ[bit 1]
"""
PCOMMIT: int = 87
"""
: CPUID.(EAX=07H, ECX=0H):EBX.PCOMMIT[bit 22]
"""
PCONFIG: int = 88
"""
: CPUID.(EAX=07H, ECX=0H):EDX.PCONFIG[bit 18]
"""
PKU: int = 89
"""
: CPUID.(EAX=07H, ECX=0H):ECX.PKU[bit 3]
"""
POPCNT: int = 90
"""
: CPUID.01H:ECX.POPCNT[bit 23]
"""
PREFETCHW: int = 91
"""
: CPUID.80000001H:ECX.PREFETCHW[bit 8]
"""
PREFETCHWT1: int = 92
"""
: CPUID.(EAX=07H, ECX=0H):ECX.PREFETCHWT1[bit 0]
"""
PTWRITE: int = 93
"""
: CPUID.(EAX=14H, ECX=0H):EBX.PTWRITE[bit 4]
"""
RDPID: int = 94
"""
: CPUID.(EAX=07H, ECX=0H):ECX.RDPID[bit 22]
"""
RDPMC: int = 95
"""
``RDPMC`` instruction (Pentium MMX or later, or Pentium Pro or later)
"""
RDPRU: int = 96
"""
: CPUID.80000008H:EBX.RDPRU[bit 4]
"""
RDRAND: int = 97
"""
: CPUID.01H:ECX.RDRAND[bit 30]
"""
RDSEED: int = 98
"""
: CPUID.(EAX=07H, ECX=0H):EBX.RDSEED[bit 18]
"""
RDTSCP: int = 99
"""
: CPUID.80000001H:EDX.RDTSCP[bit 27]
"""
RTM: int = 100
"""
: CPUID.(EAX=07H, ECX=0H):EBX.RTM[bit 11]
"""
SEP: int = 101
"""
: CPUID.01H:EDX.SEP[bit 11]
"""
SGX1: int = 102
"""
: CPUID.(EAX=12H, ECX=0H):EAX.SGX1[bit 0]
"""
SHA: int = 103
"""
: CPUID.(EAX=07H, ECX=0H):EBX.SHA[bit 29]
"""
SKINIT: int = 104
"""
: CPUID.80000001H:ECX.SKINIT[bit 12]
"""
SKINIT_OR_SVM: int = 105
"""
:class:`SKINIT` or :class:`SVM`
"""
SMAP: int = 106
"""
: CPUID.(EAX=07H, ECX=0H):EBX.SMAP[bit 20]
"""
SMX: int = 107
"""
: CPUID.01H:ECX.SMX[bit 6]
"""
SSE: int = 108
"""
: CPUID.01H:EDX.SSE[bit 25]
"""
SSE2: int = 109
"""
: CPUID.01H:EDX.SSE2[bit 26]
"""
SSE3: int = 110
"""
: CPUID.01H:ECX.SSE3[bit 0]
"""
SSE4_1: int = 111
"""
: CPUID.01H:ECX.SSE4_1[bit 19]
"""
SSE4_2: int = 112
"""
: CPUID.01H:ECX.SSE4_2[bit 20]
"""
SSE4A: int = 113
"""
: CPUID.80000001H:ECX.SSE4A[bit 6]
"""
SSSE3: int = 114
"""
: CPUID.01H:ECX.SSSE3[bit 9]
"""
SVM: int = 115
"""
: CPUID.80000001H:ECX.SVM[bit 2]
"""
SEV_ES: int = 116
"""
: CPUID.8000001FH:EAX.SEV-ES[bit 3]
"""
SYSCALL: int = 117
"""
: CPUID.80000001H:EDX.SYSCALL[bit 11]
"""
TBM: int = 118
"""
: CPUID.80000001H:ECX.TBM[bit 21]
"""
TSC: int = 119
"""
: CPUID.01H:EDX.TSC[bit 4]
"""
VAES: int = 120
"""
: CPUID.(EAX=07H, ECX=0H):ECX.VAES[bit 9]
"""
VMX: int = 121
"""
: CPUID.01H:ECX.VMX[bit 5]
"""
VPCLMULQDQ: int = 122
"""
: CPUID.(EAX=07H, ECX=0H):ECX.VPCLMULQDQ[bit 10]
"""
WAITPKG: int = 123
"""
: CPUID.(EAX=07H, ECX=0H):ECX.WAITPKG[bit 5]
"""
WBNOINVD: int = 124
"""
: CPUID.(EAX=80000008H, ECX=0H):EBX.WBNOINVD[bit 9]
"""
XOP: int = 125
"""
: CPUID.80000001H:ECX.XOP[bit 11]
"""
XSAVE: int = 126
"""
: CPUID.01H:ECX.XSAVE[bit 26]
"""
XSAVEC: int = 127
"""
: CPUID.(EAX=0DH, ECX=1H):EAX.XSAVEC[bit 1]
"""
XSAVEOPT: int = 128
"""
: CPUID.(EAX=0DH, ECX=1H):EAX.XSAVEOPT[bit 0]
"""
XSAVES: int = 129
"""
: CPUID.(EAX=0DH, ECX=1H):EAX.XSAVES[bit 3]
"""
SEV_SNP: int = 130
"""
: CPUID.8000001FH:EAX.SEV-SNP[bit 4]
"""
SERIALIZE: int = 131
"""
: CPUID.(EAX=07H, ECX=0H):EDX.SERIALIZE[bit 14]
"""
TSXLDTRK: int = 132
"""
: CPUID.(EAX=07H, ECX=0H):EDX.TSXLDTRK[bit 16]
"""
INVLPGB: int = 133
"""
: CPUID.80000008H:EBX.INVLPGB[bit 3]
"""
AMX_BF16: int = 134
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AMX-BF16[bit 22]
"""
AMX_TILE: int = 135
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AMX-TILE[bit 24]
"""
AMX_INT8: int = 136
"""
: CPUID.(EAX=07H, ECX=0H):EDX.AMX-INT8[bit 25]
"""
CYRIX_FPU: int = 137
"""
Cyrix FPU instructions (Cyrix, AMD Geode GX/LX)
"""
CYRIX_SMM: int = 138
"""
: Cyrix SMM instructions: ``SVDC``, ``RSDC``, ``SVLDT``, ``RSLDT``, ``SVTS``, ``RSTS`` (Cyrix, AMD Geode GX/LX)
"""
CYRIX_SMINT: int = 139
"""
Cyrix ``SMINT 0F38`` (6x86MX and later, AMD Geode GX/LX)
"""
CYRIX_SMINT_0F7E: int = 140
"""
Cyrix ``SMINT 0F7E`` (6x86 or earlier)
"""
CYRIX_SHR: int = 141
"""
: Cyrix SMM instructions: ``RDSHR``, ``WRSHR`` (6x86MX, M II, Cyrix III)
"""
CYRIX_DDI: int = 142
"""
: Cyrix DDI instructions: ``BB0_Reset``, ``BB1_Reset``, ``CPU_READ``, ``CPU_WRITE`` (MediaGX, GXm, GXLV, GX1)
"""
CYRIX_EMMI: int = 143
"""
: Cyrix AND CPUID.80000001H:EDX.EMMI[bit 24]
"""
CYRIX_DMI: int = 144
"""
: Cyrix DMI instructions: ``DMINT``, ``RDM`` (AMD Geode GX/LX)
"""
CENTAUR_AIS: int = 145
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.AIS[Bits 1:0] = 11B ([0] = exists, [1] = enabled)
"""
MOV_TR: int = 146
"""
MOV to/from TR (80386, Intel486, Cyrix, Geode)
"""
SMM: int = 147
"""
``RSM`` instruction (some 386s, some 486s, Pentium and later)
"""
TDX: int = 148
"""
: CPUID.(EAX=??H, ECX=?H):???.????[bit ??]
"""
KL: int = 149
"""
: CPUID.(EAX=07H, ECX=0H):ECX.KL[bit 23]
"""
AESKLE: int = 150
"""
: CPUID.19H:EBX.AESKLE[bit 0]
"""
WIDE_KL: int = 151
"""
: CPUID.19H:EBX.WIDE_KL[bit 2]
"""
UINTR: int = 152
"""
: CPUID.(EAX=07H, ECX=0H):EDX.UINTR[bit 5]
"""
HRESET: int = 153
"""
: CPUID.(EAX=07H, ECX=01H):EAX.HRESET[bit 22]
"""
AVX_VNNI: int = 154
"""
: CPUID.(EAX=07H, ECX=01H):EAX.AVX-VNNI[bit 4]
"""
PADLOCK_GMI: int = 155
"""
: CPUID.0C0000000H:EAX >= 0C0000001H AND CPUID.0C0000001H:EDX.GMI[Bits 5:4] = 11B ([4] = exists, [5] = enabled)
"""
FRED: int = 156
"""
: CPUID.(EAX=07H, ECX=01H):EAX.FRED[bit 17]
"""
LKGS: int = 157
"""
: CPUID.(EAX=07H, ECX=01H):EAX.LKGS[bit 18]
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import copy
import pytest
from iced_x86 import *

@pytest.mark.parametrize("bitness", [16, 32, 64, 0, 15, 128])
def test_invalid_bitness(bitness):
	data = b"\x90"
	if bitness == 16 or bitness == 32 or bitness == 64:
		Decoder(bitness, data)
	else:
		with pytest.raises(ValueError):
			Decoder(bitness, data)

@pytest.mark.parametrize("data, is_valid", [
	(b"", True),
	(b"\x90", True),
	(bytearray(b""), True),
	(bytearray(b"\x90"), True),
	(memoryview(b""), False),
	(memoryview(b"\x90"), False),
	(memoryview(bytearray(b"")), False),
	(memoryview(bytearray(b"\x90")), False),
	(None, False),
	(123, False),
	("Hello", False),
])
def test_different_data_types(data, is_valid):
	bitness = 64
	if is_valid:
		decoder = Decoder(bitness, data)
		if len(data) == 0:
			assert decoder.decode().code == Code.INVALID
			assert decoder.last_error == DecoderError.NO_MORE_BYTES
		else:
			assert decoder.decode().code != Code.INVALID
			assert decoder.last_error == DecoderError.NONE
	else:
		with pytest.raises(TypeError):
			Decoder(bitness, data)

@pytest.mark.parametrize("bitness, data, code", [
	(16, b"\x03\xCE", Code.ADD_R16_RM16),
	(32, b"\x03\xCE", Code.ADD_R32_RM32),
	(64, b"\x48\x03\xCE", Code.ADD_R64_RM64),
])
def test_bitness(bitness, data, code):
	decoder = Decoder(bitness, data)
	assert decoder.bitness == bitness
	instr = decoder.decode()
	assert instr.code == code

def test_without_decoder_options():
	decoder = Decoder(64, b"\x0F\x1A\x08")
	instr = decoder.decode()
	assert instr.code != Code.BNDLDX_BND_MIB

def test_with_decoder_options():
	decoder = Decoder(64, b"\x0F\x1A\x08", DecoderOptions.MPX)
	instr = decoder.decode()
	assert instr.code == Code.BNDLDX_BND_MIB

def test_iter():
	decoder = Decoder(64, b"\x48\x09\xCE\x90\xF3\x90")
	instrs = [instr for instr in decoder]
	assert len(instrs) == 3
	assert instrs[0].code == Code.OR_RM64_R64
	assert instrs[1].code == Code.NOPD
	assert instrs[2].code == Code.PAUSE

def test_ip():
	decoder = Decoder(64, b"\x48\x09\xCE\x90")
	assert decoder.ip == 0
	decoder = Decoder(64, b"\x48\x09\xCE\x90", ip=0xABCD_EF01_2345_6789)
	assert decoder.ip == 0xABCD_EF01_2345_6789
	decoder.ip = 0x1234_5678_9ABC_DEF0
	assert decoder.ip == 0x1234_5678_9ABC_DEF0
	decoder.decode()
	assert decoder.ip == 0x1234_5678_9ABC_DEF3
	decoder.decode()
	assert decoder.ip == 0x1234_5678_9ABC_DEF4
	decoder.decode()
	assert decoder.ip == 0x1234_5678_9ABC_DEF4
	decoder.ip = 0xFEDC_BA98_7654_3210
	assert decoder.ip == 0xFEDC_BA98_7654_3210

def test_position():
	decoder = Decoder(64, b"\x48\x09\xCE\x90\xF3\x90")

	for _ in range(2):
		assert decoder.can_decode
		assert decoder.max_position == 6
		assert decoder.position == 0

		assert decoder.decode().code == Code.OR_RM64_R64
		assert decoder.can_decode
		assert decoder.max_position == 6
		assert decoder.position == 3

		assert decoder.decode().code == Code.NOPD
		assert decoder.can_decode
		assert decoder.max_position == 6
		assert decoder.position == 4

		assert decoder.decode().code == Code.PAUSE
		assert not decoder.can_decode
		assert decoder.max_position == 6
		assert decoder.position == 6

		assert decoder.decode().code == Code.INVALID
		assert not decoder.can_decode
		assert decoder.max_position == 6
		assert decoder.position == 6

		decoder.position = 0

	decoder.position = 3
	assert decoder.decode().code == Code.NOPD
	assert decoder.can_decode
	assert decoder.max_position == 6
	assert decoder.position == 4

def test_invalid_position():
	decoder = Decoder(64, b"\x48\x09\xCE\x90\xF3\x90")
	decoder.position = 0
	decoder.position = 6
	with pytest.raises(ValueError):
		decoder.position = 7

def test_last_error():
	decoder = Decoder(64, b"\x90\x48")
	assert decoder.last_error == DecoderError.NONE

	assert decoder.decode().code == Code.NOPD
	assert decoder.last_error == DecoderError.NONE
	assert decoder.decode().code == Code.INVALID
	assert decoder.last_error == DecoderError.NO_MORE_BYTES

	decoder = Decoder(64, b"\xF0\x90")
	assert decoder.decode().code == Code.INVALID
	assert decoder.last_error == DecoderError.INVALID_INSTRUCTION

def test_decode():
	data = b"\x48\x09\xCE\x90\xF3\x90"
	decodera = Decoder(64, data)
	decoderb = Decoder(64, data)
	instr = Instruction()

	instr1 = decodera.decode()
	decoderb.decode_out(instr)
	assert instr.eq_all_bits(instr1)

	instr2 = decodera.decode()
	decoderb.decode_out(instr)
	assert instr.eq_all_bits(instr2)

def test_offsets():
	decoder = Decoder(64, b"\x90\x83\xB3\x34\x12\x5A\xA5\x5A")

	instr = decoder.decode()
	co = decoder.get_constant_offsets(instr)
	assert co.displacement_offset == 0
	assert co.displacement_size == 0
	assert co.immediate_offset == 0
	assert co.immediate_size == 0
	assert co.immediate_offset2 == 0
	assert co.immediate_size2 == 0
	assert not co.has_displacement
	assert not co.has_immediate
	assert not co.has_immediate2

	instr = decoder.decode()
	co = decoder.get_constant_offsets(instr)
	assert co.displacement_offset == 2
	assert co.displacement_size == 4
	assert co.immediate_offset == 6
	assert co.immediate_size == 1
	assert co.immediate_offset2 == 0
	assert co.immediate_size2 == 0
	assert co.has_displacement
	assert co.has_immediate
	assert not co.has_immediate2

def test_co_eq_ne_hash():
	decoder = Decoder(64, b"\x90\x90\x83\xB3\x34\x12\x5A\xA5\x5A\x83\xB3\x34\x12\x5A\xA5\x5A")
	instr = decoder.decode()
	co1 = decoder.get_constant_offsets(instr)
	instr = decoder.decode()
	co2 = decoder.get_constant_offsets(instr)
	instr = decoder.decode()
	co3 = decoder.get_constant_offsets(instr)
	instr = decoder.decode()
	co4 = decoder.get_constant_offsets(instr)

	assert id(co1) != id(co2)
	assert id(co3) != id(co4)

	assert hash(co1) == hash(co2)
	assert hash(co3) == hash(co4)

	assert co1 == co2
	assert not (co1 != co2)

	assert co3 == co4
	assert not (co3 != co4)

	assert co1 != co3
	assert not (co1 == co3)

	assert co1 != 1
	assert co1 != 1.23
	assert co1 != None
	assert co1 != []
	assert co1 != {}
	assert co1 != (1, 2)

	assert not (co1 == 1)
	assert not (co1 == 1.23)
	assert not (co1 == None)
	assert not (co1 == [])
	assert not (co1 == {})
	assert not (co1 == (1, 2))

@pytest.mark.parametrize("copy_co", [
	lambda instr: copy.copy(instr),
	lambda instr: copy.deepcopy(instr),
	lambda instr: instr.copy(),
])
def test_co_copy_deepcopy_mcopy(copy_co):
	decoder = Decoder(64, b"\x90\x83\xB3\x34\x12\x5A\xA5\x5A", ip=0x1234_5678_9ABC_DEF1)
	coa = decoder.get_constant_offsets(decoder.decode())
	cob = decoder.get_constant_offsets(decoder.decode())

	coa2 = copy_co(coa)
	assert coa is not coa2
	assert id(coa) != id(coa2)
	assert coa == coa2

	cob2 = copy_co(cob)
	assert cob is not cob2
	assert id(cob) != id(cob2)
	assert cob == cob2

def test_no_bytes():
	decoder = Decoder(64, b"")
	assert not decoder.can_decode
	assert decoder.position == 0
	assert decoder.max_position == 0

	instr = decoder.decode()
	assert decoder.last_error == DecoderError.NO_MORE_BYTES
	assert instr.is_invalid
	assert not instr
	assert not decoder.can_decode
	assert decoder.position == 0
	assert decoder.max_position == 0

def test_no_invalid_check_option():
	data = b"\xF0\x02\xCE"
	decodera = Decoder(64, data)
	decoderb = Decoder(64, data, DecoderOptions.NO_INVALID_CHECK)

	instra = decodera.decode()
	instrb = decoderb.decode()

	assert instra.code == Code.INVALID
	assert decodera.last_error == DecoderError.INVALID_INSTRUCTION
	assert instrb.code == Code.ADD_R8_RM8

def test_amd_option():
	data = b"\x66\x70\x5A"
	decodera = Decoder(64, data)
	decoderb = Decoder(64, data, DecoderOptions.AMD)

	instra = decodera.decode()
	instrb = decoderb.decode()

	assert instra.code == Code.JO_REL8_64
	assert instrb.code == Code.JO_REL8_16

def test_multiple_options():
	decoder = Decoder(64, b"\xF3\x90", DecoderOptions.AMD | DecoderOptions.NO_PAUSE)
	instr = decoder.decode()
	assert instr.code == Code.NOPD



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
``RFLAGS`` bits, FPU condition code bits and misc bits (``UIF``) supported by the instruction info code
"""

NONE: int = 0x0000_0000
"""
No bit is set
"""
OF: int = 0x0000_0001
"""
``RFLAGS.OF``
"""
SF: int = 0x0000_0002
"""
``RFLAGS.SF``
"""
ZF: int = 0x0000_0004
"""
``RFLAGS.ZF``
"""
AF: int = 0x0000_0008
"""
``RFLAGS.AF``
"""
CF: int = 0x0000_0010
"""
``RFLAGS.CF``
"""
PF: int = 0x0000_0020
"""
``RFLAGS.PF``
"""
DF: int = 0x0000_0040
"""
``RFLAGS.DF``
"""
IF: int = 0x0000_0080
"""
``RFLAGS.IF``
"""
AC: int = 0x0000_0100
"""
``RFLAGS.AC``
"""
UIF: int = 0x0000_0200
"""
``UIF``
"""
C0: int = 0x0000_0400
"""
FPU status word bit ``C0``
"""
C1: int = 0x0000_0800
"""
FPU status word bit ``C1``
"""
C2: int = 0x0000_1000
"""
FPU status word bit ``C2``
"""
C3: int = 0x0000_2000
"""
FPU status word bit ``C3``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JE`` / ``JZ``)
"""

E: int = 0
"""
``JE``, ``CMOVE``, ``SETE``, ``LOOPE``, ``REPE``
"""
Z: int = 1
"""
``JZ``, ``CMOVZ``, ``SETZ``, ``LOOPZ``, ``REPZ``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import copy
import pytest
from iced_x86 import *

def test_info():
	instr = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41").decode()
	factory = InstructionInfoFactory()
	info = factory.info(instr)

	assert info.op0_access == OpAccess.WRITE
	assert info.op1_access == OpAccess.READ
	assert info.op2_access == OpAccess.READ
	assert info.op3_access == OpAccess.READ
	assert info.op4_access == OpAccess.READ
	assert info.op_access(0) == info.op0_access
	assert info.op_access(1) == info.op1_access
	assert info.op_access(2) == info.op2_access
	assert info.op_access(3) == info.op3_access
	assert info.op_access(4) == info.op4_access

	mem_list = info.used_memory()
	assert type(mem_list) == list
	assert len(mem_list) == 1
	mem = mem_list[0]
	assert mem.segment == Register.DS
	assert mem.base == Register.RAX
	assert mem.index == Register.NONE
	assert mem.scale == 1
	assert mem.displacement == 0
	assert mem.displacement_i64 == 0
	assert mem.memory_size == MemorySize.PACKED128_FLOAT32
	assert mem.access == OpAccess.READ
	assert mem.address_size == CodeSize.CODE64
	assert mem.vsib_size == 0

	reg_list = info.used_registers()
	assert type(reg_list) == list
	assert len(reg_list) == 4
	assert (reg_list[0].register, reg_list[0].access) == (Register.ZMM2, OpAccess.WRITE)
	assert (reg_list[1].register, reg_list[1].access) == (Register.XMM6, OpAccess.READ)
	assert (reg_list[2].register, reg_list[2].access) == (Register.RAX, OpAccess.READ)
	assert (reg_list[3].register, reg_list[3].access) == (Register.XMM4, OpAccess.READ)

@pytest.mark.parametrize("copy_value", [
	lambda instr: copy.copy(instr),
	lambda instr: copy.deepcopy(instr),
	lambda instr: instr.copy(),
])
def test_copy_deepcopy_mcopy(copy_value):
	instr = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41").decode()
	factory = InstructionInfoFactory()
	info = factory.info(instr)

	mem = info.used_memory()[0]
	mem2 = copy_value(mem)
	assert mem is not mem2
	assert id(mem) != id(mem2)
	assert mem == mem2
	assert not (mem != mem2)
	assert hash(mem) == hash(mem2)

	reg = info.used_registers()[0]
	reg2 = copy_value(reg)
	assert reg is not reg2
	assert id(reg) != id(reg2)
	assert reg == reg2
	assert not (reg != reg2)
	assert hash(reg) == hash(reg2)

def test_op_access_raise():
	instr = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41").decode()
	factory = InstructionInfoFactory()
	info = factory.info(instr)

	info.op_access(4)
	with pytest.raises(ValueError):
		info.op_access(123)



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JLE`` / ``JNG``)
"""

LE: int = 0
"""
``JLE``, ``CMOVLE``, ``SETLE``
"""
NG: int = 1
"""
``JNG``, ``CMOVNG``, ``SETNG``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Control flow
"""

NEXT: int = 0
"""
The next instruction that will be executed is the next instruction in the instruction stream
"""
UNCONDITIONAL_BRANCH: int = 1
"""
: It's an unconditional branch instruction: ``JMP NEAR``, ``JMP FAR``
"""
INDIRECT_BRANCH: int = 2
"""
: It's an unconditional indirect branch: ``JMP NEAR reg``, ``JMP NEAR [mem]``, ``JMP FAR [mem]``
"""
CONDITIONAL_BRANCH: int = 3
"""
: It's a conditional branch instruction: ``Jcc SHORT``, ``Jcc NEAR``, ``LOOP``, ``LOOPcc``, ``JRCXZ``
"""
RETURN: int = 4
"""
: It's a return instruction: ``RET NEAR``, ``RET FAR``, ``IRET``, ``SYSRET``, ``SYSEXIT``, ``RSM``, ``SKINIT``, ``RDM``, ``UIRET``
"""
CALL: int = 5
"""
: It's a call instruction: ``CALL NEAR``, ``CALL FAR``, ``SYSCALL``, ``SYSENTER``, ``VMLAUNCH``, ``VMRESUME``, ``VMCALL``, ``VMMCALL``, ``VMGEXIT``, ``VMRUN``, ``TDCALL``, ``SEAMCALL``, ``SEAMRET``
"""
INDIRECT_CALL: int = 6
"""
: It's an indirect call instruction: ``CALL NEAR reg``, ``CALL NEAR [mem]``, ``CALL FAR [mem]``
"""
INTERRUPT: int = 7
"""
: It's an interrupt instruction: ``INT n``, ``INT3``, ``INT1``, ``INTO``, ``SMINT``, ``DMINT``
"""
XBEGIN_XABORT_XEND: int = 8
"""
It's ``XBEGIN``
"""
EXCEPTION: int = 9
"""
It's an invalid instruction, eg. :class:`iced_x86.Code.INVALID`, ``UD0``, ``UD1``, ``UD2``
"""
